// $Id: bignum.h 762 2009-12-07 23:22:39Z mag $
#ifndef BITSET_H_
#define BITSET_H_

#include <memory.h>

template <uint8_t B>
class bitset
{

    uint8_t _v[B];

public:

    bitset();
    bitset(const bitset<B>& b);

    bitset& set(uint8_t i);
    bitset& reset(uint8_t i);

    uint8_t count() const;

    bool test(uint8_t i) const;

};

template <uint8_t B>
bitset<B>::bitset()
{
    memset(_v,0,B*sizeof(uint8_t));
}

template <uint8_t B>
bitset<B>::bitset(const bitset<B>& b)
{
    memcpy(_v,b._v,B*sizeof(uint8_t));
}

template <uint8_t B>
bitset<B>&
bitset<B>::set(uint8_t i)
{
    register uint8_t b = i >> 3;
    register uint8_t m = i & 0x7;
    _v[b] |= 1 << m;
    return *this;
}

template <uint8_t B>
bitset<B>&
bitset<B>::reset(uint8_t i)
{
    register uint8_t b = i >> 3;
    register uint8_t m = i & 0x7;
    _v[b] &= ~(1 << m);
    return *this;
}

template <uint8_t B>
uint8_t
bitset<B>::count() const
{
    uint8_t c = 0;
    static const uint8_t M = B << 3;
    for (uint8_t i = 0; i < M; ++i)
        if (test(i))
            ++c;
    return c;
}

template <uint8_t B>
bool
bitset<B>::test(uint8_t i) const
{
    register uint8_t b = i >> 3;
    register uint8_t m = i & 0x7;
    return (_v[b] & (1 << m)) != 0;
}

#endif /*BITSET_H_*/
