// $Id: counter.h 758 2009-11-27 01:00:19Z mag $
#ifndef COUNTER_H_
#define COUNTER_H_

#include "tuple.h"
#include "sample.h"

template <uint8_t N>
class Counter {

    uint64_t _i;

    static uint64_t n;
    static Sample<N>* smp;
    static Tuple<N,char>* idx;

public:

    Counter();

    bool finished() const;

    Counter& reset(bool adv);

    Counter& operator ++ (int);

    const Sample<N>& sample_offset() const;
    const Tuple<N,char>& index_offset() const;

    static void init();

};

template <uint8_t N>
uint64_t
Counter<N>::n = 1ULL;

template <uint8_t N>
Tuple<N,char>*
Counter<N>::idx = 0;

template <uint8_t N>
Sample<N>*
Counter<N>::smp = 0;

template <uint8_t N>
inline
Counter<N>::Counter() :
    _i(0ULL) {}

template <uint8_t N>
inline Counter<N>&
Counter<N>::reset(bool adv)
{ _i = (adv) ? 1ULL : 0ULL; return *this; }

template <uint8_t N>
inline bool
Counter<N>::finished() const
{ return _i == n; }

template <uint8_t N>
inline const Tuple<N,char>&
Counter<N>::index_offset() const
{ return idx[_i]; }

template <uint8_t N>
inline const Sample<N>&
Counter<N>::sample_offset() const
{ return smp[_i]; }

template <uint8_t N>
inline Counter<N>&
Counter<N>::operator ++ (int)
{ _i++; return *this; }

template <uint8_t N>
void
Counter<N>::init()
{

    uint8_t i;

    for (i = 0; i < N; i++)
        n *= 3ULL;

    smp = new Sample<N>[n];
    idx = new Tuple<N,char>[n];

    int8_t c[N];

    for (i = 0; i < N; i++)
        c[i] = 0;

    for (uint64_t j = 0; j < n; j++) {

        uint8_t k;

        for (k = 0; k < N; k++) {
            smp[j].get(k) = static_cast<float>((c[k] + 1)%3 - 1);
            idx[j].get(k) = (c[k] + 1)%3 - 1;
        }

        for (k = 0; k < N; k++) {

            c[k] += 1;

            if (c[k] > 2)
                c[k] = 0;
            else
                break;

        }

    }

}

#endif /*COUNTER_H_*/
