// $Id: cube.h 763 2010-01-28 23:15:33Z mag $
#ifndef CUBE_H_
#define CUBE_H_

#include <cassert>

#include "tuple.h"
#include "sample.h"
#include "interval.h"

template <uint8_t N>
class Cube: public Tuple<N,Interval> {

    Tuple<N,double> _m;

    void midpoint();

    static const uint8_t M = 1 << N;

public:

    Cube();
    Cube(const Tuple<N,Interval>& t);

    Sample<N> randomise() const;

    Cube<N> child(uint8_t i) const;

    void subdivide(Cube<N>* c) const;

    bool contains(const Sample<N>& s) const;

    friend bool operator == (const Cube<N>& c1,
                             const Cube<N>& c2)
    { const Tuple<N,Interval>& t1 = static_cast< const Tuple<N,Interval>& >(c1);
      const Tuple<N,Interval>& t2 = static_cast< const Tuple<N,Interval>& >(c2);
      return t1 == t2; }

};

template <uint8_t N>
inline
Cube<N>::Cube()
{
    midpoint();
}

template <uint8_t N>
inline
Cube<N>::Cube(const Tuple<N,Interval>& t) :
    Tuple<N,Interval>(t)
{
    midpoint();
}

template <uint8_t N>
void
Cube<N>::midpoint()
{

    for (uint8_t i = 0; i < N; i++)
        _m.get(i) = Tuple<N,Interval>::get(i).midpoint();

}

template <uint8_t N>
Cube<N>
Cube<N>::child(uint8_t i) const
{

    Cube<N> c;

    assert(i < M);

    for (uint8_t j = 0; j < N; j++) {

        if (i % 2)
            c.get(j) = Interval(_m.get(j),Tuple<N,Interval>::get(j).max());
        else
            c.get(j) = Interval(Tuple<N,Interval>::get(j).min(),_m.get(j));

        i >>= 1;

    }

    c.midpoint();

    return c;

}

template <uint8_t N>
void
Cube<N>::subdivide(Cube<N>* c) const
{

    Cube<N>* p = c;

    for (uint8_t i = 0; i < M; i++) {

        uint8_t j = i;

        for (uint8_t k = 0; k < N; k++) {

            if (j % 2)
                p->get(k) = Interval(_m.get(k),Tuple<N,Interval>::get(k).max());
            else
                p->get(k) = Interval(Tuple<N,Interval>::get(k).min(),_m.get(k));

            j >>= 1;

        }

        p->midpoint();
        p++;

    }

}

template <uint8_t N>
Sample<N>
Cube<N>::randomise() const
{

    Sample<N> s;

    for (uint8_t i = 0; i < N; i++)
        s.get(i) = Tuple<N,Interval>::get(i).randomise();

    return s;

}

template <uint8_t N>
bool
Cube<N>::contains(const Sample<N>& s) const
{

    for (uint8_t i = 0; i < N; i++)
        if (!Tuple<N,Interval>::get(i).contains(s.get(i)))
            return false;

    return true;

}

#endif
