// $Id: tuple.h 761 2009-11-29 12:16:21Z mag $
#ifndef TUPLE_H_
#define TUPLE_H_

#include <vector>
#include <cassert>
#include <iostream>

#include "interval.h"

using namespace std;

template <uint8_t N, typename T>
class Tuple {

    T _v[N];

public:

    Tuple();
    Tuple(const T& x);

    T& get(uint8_t i);
    const T& get(uint8_t i) const;

    Tuple operator - () const;

    template <typename U>
    Tuple& operator += (const Tuple<N,U>& x);

    bool operator == (const Tuple<N,T>& t) const;

};

template <uint8_t N, typename T>
inline
Tuple<N,T>::Tuple() {}

template <uint8_t N, typename T>
inline
Tuple<N,T>::Tuple(const T& x)
{ for (uint8_t i = 0; i < N; i++) _v[i] = x; }

template <uint8_t N, typename T>
inline T&
Tuple<N,T>::get(uint8_t i)
{ assert(i < N); return _v[i]; }

template <uint8_t N, typename T>
inline const T&
Tuple<N,T>::get(uint8_t i) const
{ assert(i < N); return _v[i]; }

template <uint8_t N, typename T>
Tuple<N,T>
Tuple<N,T>::operator - () const
{ Tuple<N,T> t;
  for (uint8_t i = 0; i < N; i++) t._v[i] = -_v[i];
  return t; }

template <uint8_t N, typename T>
template <typename U>
inline Tuple<N,T>&
Tuple<N,T>::operator += (const Tuple<N,U>& x)
{ for (uint8_t i = 0; i < N; i++) _v[i] += x.get(i);
  return *this; }

template <uint8_t N, typename T>
bool
Tuple<N,T>::operator == (const Tuple<N,T>& t) const
{

  for (uint8_t i = 0; i < N; i++)
    if (_v[i] != t._v[i])
      return false;

  return true;

}

template <uint8_t N, typename T>
ostream&
operator << (ostream& s, const Tuple<N,T>& x)
{

    s << '(';

    for (uint8_t i = 0; i < N; i++) {
        s << x.get(i);
        if (i == N - 1)
            break;
        s << ',';
    }

    return s << ')';

}

#endif /*TUPLE_H_*/
